var path = require('path');
var express = require('express');
var bodyParser = require('body-parser');
var request = require('request');

var app = express();

var config = {
  port: 8787,
  apiEndpoint: '/api',
  apiPassThroughUrl: null
}

var staticOptions = {
  'dotfiles': 'deny',
  'etag': true,
  'extensions': false,
  'index': false,
  'lastModified': true,
  'maxAge': 0,
  'redirect': false,
  'setHeaders': function(response, path, stat) {
  }
};

// Current Working Directory
var cwd = path.resolve('.');
var angularAppStatic = express.static(path.join(cwd, 'app'));
var publicStatic = express.static(path.join(cwd, 'public'));

// Check for API pass-through url
var args = process.argv.slice(2);
if (args[0] === '--api') {
  config.apiPassThroughUrl = args[1];
}

console.log('Angular Dev Server started on port', config.port);
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }));
app.use(publicStatic);
app.use(angularAppStatic);

if (config.apiPassThroughUrl !== null) {
  app.use(config.apiEndpoint, function(req, res, next) {
    console.log('api request url: %s %s', req.method, req.originalUrl);
    var options = {
      uri: config.apiPassThroughUrl + req.originalUrl.slice(
	req.originalUrl.indexOf(config.apiEndpoint) +
	  config.apiEndpoint.length + 1
      ),
      method: req.method,
      qs: req.query
    };

    if (req.method === 'POST') {
      options.form = req.body.params;
    }

    console.log('api proxy request: %s %s', options.method, options.uri);
    request(options, function(error, response, body) {
      if (error) {
	console.error('api proxy error: %s', error);
      }
      if (response.statusCode) {
	res.status(response.statusCode);
      }
      res.send(body);
    });
  });
}

app.listen(config.port);
